/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	ddialogf.c:
	Dialogf functions for editing booleans as check boxes.

	By Sven Sandberg

	See readme.txt for more information about Dime.


	int my_int;
	dialogf(..., "Description%dialogf[Open dialog]",
			  "Title of sub-dialog", x, y, edit_w,
			  "New format string%bool[]", &my_int);
*/
#include <stdarg.h>
#include <allegro.h>

#ifdef FORTIFY
#include "fortify2.h"
#endif

#include "dime.h"
#include "dime/dimeint.h"



/* dialogf_create_dialogf:
*/
static void dialogf_create_dialogf_ex(DIALOG *d, char *arg_text,
												  dime_va_list_p args, int v)
{
	char *title;
	int x, y, w;
	char *format;
	int state;
	char buf[4096];
	#ifdef ALLEGRO_WATCOM
		/* Watcom uses the definition char *va_list[] so it can't be an lvalue
		*/
		void *sub_args;
	#else
		va_list sub_args;
	#endif

	/* Parse parameters. */
	state = dime_get_string_arg(&arg_text, buf, sizeof(buf));
	DIME_ASSERT(state & DIME_ARG_END,
					("Too many arguments in format modifier for %%dialogf."));

	d->proc = d_dime_button_proc;
	d->flags |= D_EXIT;
	d->h += 6;
	d->dp = ustrdup(buf);

	title = dime_va_arg_p(args, char *);
	x = dime_va_arg_p(args, int);
	y = dime_va_arg_p(args, int);
	w = dime_va_arg_p(args, int);
	format = dime_va_arg_p(args, char *);

	if (v) {
		sub_args = dime_va_arg_p(args, va_list);
		d->dp3 = dialogf_start(title, x, y, w, format,
									  (dime_va_list_p)&sub_args);
	}
	else
		d->dp3 = dialogf_start(title, x, y, w, format, args);
}



/* dialogf_create_dialogf:
*/
int dialogf_create_dialogf(DIALOG *d, char *desc, char *arg_text,
									dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	dialogf_create_dialogf_ex(d, arg_text, args, FALSE);
	return 0;
}



/* dialogf_create_vdialogf:
*/
int dialogf_create_vdialogf(DIALOG *d, char *desc, char *arg_text,
									 dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	dialogf_create_dialogf_ex(d, arg_text, args, TRUE);
	return 0;
}



/* dialogf_outside_dialogf:
*/
int dialogf_outside_dialogf(DIALOG *d)
{
	if (dialogf_run((DIALOGF_DIALOG *)d->dp3) == 1)
		dialogf_store_results((DIALOGF_DIALOG *)d->dp3);
	else
		dialogf_reset_dialog((DIALOGF_DIALOG *)d->dp3);

	return 0;
}



/* dialogf_store_dialogf:
*/
void dialogf_store_dialogf(DIALOG *d)
{
	dialogf_store_results((DIALOGF_DIALOG *)d->dp3);
}



/* dialogf_reset_dialogf:
*/
void dialogf_reset_dialogf(DIALOG *d)
{
	dialogf_reset_dialog((DIALOGF_DIALOG *)d->dp3);
}



/* dialogf_destroy_dialogf:
*/
void dialogf_destroy_dialogf(DIALOG *d)
{
	dialogf_end((DIALOGF_DIALOG *)d->dp3);
	free(d->dp);
}